@@ -1,12 +1,12 @@
 ;;; develock.el --- additional font-lock keywords for the developers
 
-;; Copyright (C) 2001, 2002, 2003 Katsumi Yamaoka
+;; Copyright (C) 2001, 2002, 2003, 2005 Katsumi Yamaoka
 
 ;; Author: Katsumi Yamaoka  <yamaoka@jpl.org>
 ;;         Jun'ichi Shiono  <jun@fsas.fujitsu.com>
 ;;         Yasutaka SHINDOH <ring-pub@fan.gr.jp>
 ;; Created: 2001/06/28
-;; Revised: 2003/11/13
+;; Revised: 2005/03/04
 ;; Keywords: font-lock emacs-lisp change-log texinfo c java perl html
 ;;           mail news
 
@@ -87,8 +87,15 @@
 ;; =======
 ;; The value of `fill-column' for `texinfo-mode' has changed to 70 in
 ;; Emacs 21.  However, many texinfo files are written on the condition
-;; that the value is 72.  If too much highlighting offends your eyes,
-;; you can change the behavior of highlighting as shown below:
+;; that the value is 72.  To against it, Develock provides the
+;; `develock-fill-column-alist' variable which makes it possible to
+;; set the value for `fill-column' automatically.  If Develock is on,
+;; it is set to 72 and also the `auto-fill-mode' is enabled in the
+;; `texinfo-mode' by default.
+;;
+;; Even if you disable it in `develock-fill-column-alist', you can
+;; change the behavior of highlighting if too much highlighting
+;; offends your eyes, as shown below:
 ;;
 ;;(if (and (not (featurep 'xemacs))
 ;;	 (>= emacs-major-version 21))
@@ -202,7 +209,7 @@
 
 ;;; Code:
 
-(defconst develock-version "0.23"
+(defconst develock-version "0.25"
   "Version number for this version of Develock.")
 
 (require 'advice)
@@ -217,10 +224,10 @@
    "Internal buffer-local variable used to indicate whether Develock is on."))
 
 (defcustom develock-mode-strings '(" Develock" " Font")
-  "List of identifications to put in the modeline whether Develock mode
-is on or off.  Each element should be a string beginning with a space,
-a null string or a symbol with a similar string as its value.  You can
-use faces for each element if you are using Emacs 21 as follows:
+  "List of identifications used to show whether Develock is on or off.
+Each element should be a string beginning with a space, a null string
+or a symbol with a similar string as its value.  You can use faces for
+each element if you are using Emacs 21 as follows:
 
 \(make-face 'develock-mode)
 \(setq develock-mode-strings
@@ -316,8 +323,8 @@ minor mode strings as shown below:
      (:foreground "Black" :background "Yellow"))
     (t
      (:foreground "Yellow" :background "Black")))
-  "Develock face used to highlight characters (or words) which violate
-the manners of mail messages or news articles."
+  "Develock face used to highlight unwanted characters in messages.
+Those might violate the manners of mail messages or news articles."
   :group 'develock
   :group 'font-lock-highlighting-faces
   :group 'font-lock-faces)
@@ -344,8 +351,8 @@ the manners of mail messages or news articles."
 
 (defface develock-reachable-mail-address-face
   '((t (:foreground "DarkGreen" :background "LemonChiffon")))
-  "Develock face used to highlight reachable E-mail addresses.  That
-would be defenseless to SPAMMERS.")
+  "Develock face used to highlight reachable E-mail addresses.
+That would be defenseless to spammers.")
 
 (defcustom develock-auto-enable t
   "If non-nil, turn Develock mode on when font-lock is turned on."
@@ -367,8 +374,7 @@ would be defenseless to SPAMMERS.")
 	(setq buffers (cdr buffers))))))
 
 (defun develock-custom-set-and-refontify (symbol value)
-  "Function used to the :set keyword for the Develock options which
-affect buffer's representation."
+  "Function used to refontify buffers when customizing Develock options."
   (prog1
       (if (fboundp 'custom-set-default)
 	  (custom-set-default symbol value)
@@ -393,10 +399,12 @@ affect buffer's representation."
 	'mail-mode t
 	'message-mode t
 	'cmail-mail-mode t)
-  "Plist of `major-mode's and limitations that lines should not be longer
-than the value.  Value `w' means one subtracted from the window width.
-Value t means the return value of `current-fill-column'.  You can
-inhibit to highlight long lines using the value nil."
+  "Plist of `major-mode's and limitation values for long lines.
+The part of a line that is longer than the limitation value according
+to the `major-mode' is highlighted.  Value `w' means one subtracted
+from the window width.  Value t means the return value of
+`current-fill-column'.  You can inhibit to highlight long lines using
+the value nil."
   :type '(repeat
 	  (list :indent 2 :inline t :format "%v"
 		(symbol :format "Major-Mode: %v\n" :size 0)
@@ -409,6 +417,19 @@ inhibit to highlight long lines using the value nil."
   :group 'develock
   :group 'font-lock)
 
+(defcustom develock-fill-column-alist
+  '((change-log-mode . 74)
+    (texinfo-mode . 72))
+  "Alist of `major-mode's and the values for `fill-column'.
+When Develock is turned on, `auto-fill-mode' is automatically enabled
+in the buffer where `major-mode' included in this list runs.
+Each value is applied to `fill-column' in the buffer."
+  :type '(repeat (cons :format "%v"
+		       (symbol :tag "Major mode")
+		       (integer :tag "Fill column")))
+  :group 'develock
+  :group 'font-lock)
+
 (defcustom develock-mode-ignore-kinsoku-list
   '(emacs-lisp-mode lisp-interaction-mode c-mode c++-mode java-mode jde-mode
 		    cperl-mode perl-mode)
@@ -497,48 +518,37 @@ try the following advice in your startup file.
     (defalias 'develock-point-at-eol 'line-end-position)))
 
 (defvar lisp-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Lisp modes.")
+  "Extraordinary level font-lock keywords for the Lisp modes.")
 
 (defvar change-log-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Change Log mode.")
+  "Extraordinary level font-lock keywords for the Change Log mode.")
 
 (defvar texinfo-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the TeXinfo mode.")
+  "Extraordinary level font-lock keywords for the TeXinfo mode.")
 
 (defvar c-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the C modes.")
+  "Extraordinary level font-lock keywords for the C modes.")
 
 (defvar java-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Java mode.")
+  "Extraordinary level font-lock keywords for the Java mode.")
 
 (defvar cperl-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the CPerl mode.")
+  "Extraordinary level font-lock keywords for the CPerl mode.")
 
 (defvar perl-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Perl mode.")
+  "Extraordinary level font-lock keywords for the Perl mode.")
 
 (defvar mail-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Mail mode.")
+  "Extraordinary level font-lock keywords for the Mail mode.")
 
 (defvar message-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Message mode.")
+  "Extraordinary level font-lock keywords for the Message mode.")
 
 (defvar cmail-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the Cmail mode.")
+  "Extraordinary level font-lock keywords for the Cmail mode.")
 
 (defvar html-font-lock-keywords-x nil
-  "Internal variable used to have extraordinary level font-lock keywords
-for the HTML mode.")
+  "Extraordinary level font-lock keywords for the HTML mode.")
 
 (defvar develock-keywords-alist
   '((emacs-lisp-mode lisp-font-lock-keywords-x
@@ -571,8 +581,8 @@ for the HTML mode.")
 		  develock-mail-font-lock-keywords)
     (cmail-mail-mode cmail-font-lock-keywords-x
 		     develock-mail-font-lock-keywords))
-  "*Alist of keyword symbols for major modes.  Each element should be
-triple symbols of the following form:
+  "*Alist of keyword symbols for major modes.
+Each element should be triple symbols of the following form:
 
 \(major-mode internal-keywords user-defined-keywords)")
 
@@ -1079,10 +1089,15 @@ triple symbols of the following form:
        ,form)))
 
 (defvar develock-inhibit-advice-font-lock-set-defaults nil
-  "Internal variable used to inhibit running the advised function
-definition of `font-lock-set-defaults' or `font-lock-set-defaults-1'
-if it is non-nil.  It will be set to t when the function is called
-recursively by itself.  Users should never modify the value.")
+  "Non-nil means inhibit advising of the `font-lock-set-defaults' function.
+In XEmacs, it is the `font-lock-set-defaults-1' function instead.
+It will be set to t when the function is called recursively by itself.
+Users should never modify the value.")
+
+(defvar develock-original-fill-configuration nil
+  "Internal variable used to save `fill-column' and `auto-fill-mode'.
+It becomes buffer-local in the buffer in which Develock is on, and
+keeps the values as a cons cell before Develock is turned on.")
 
 (let (current-load-list)
   (eval
@@ -1100,6 +1115,7 @@ Energize font-lock keywords for extraordinary level highlighting."
 	      (let* ((defaults (develock-find-font-lock-defaults major-mode))
 		     (keywords (delq (car def) (car defaults)))
 		     (max-decor (nth (1- (length keywords)) keywords))
+		     (oldmode develock-mode)
 		     (font-lock-maximum-decoration
 		      font-lock-maximum-decoration))
 		(setq develock-mode
@@ -1116,7 +1132,7 @@ Energize font-lock keywords for extraordinary level highlighting."
 				  (file-name-nondirectory buffer-file-name))))
 			   develock-auto-enable))
 		(if develock-mode
-		    (progn
+		    (let (fc)
 		      (setq font-lock-maximum-decoration t)
 		      (set (car def)
 			   (append (if (fboundp max-decor)
@@ -1124,7 +1140,18 @@ Energize font-lock keywords for extraordinary level highlighting."
 				     (symbol-value max-decor))
 				   (symbol-value (car (cdr def)))))
 		      (setcdr (nthcdr (1- (length keywords)) keywords)
-			      (list (car def)))))
+			      (list (car def)))
+		      (if (and (not oldmode)
+			       (setq fc
+				     (cdr (assq major-mode
+						develock-fill-column-alist))))
+			  (progn
+			    (set (make-local-variable
+				  'develock-original-fill-configuration)
+				 (cons (current-fill-column)
+				       (and auto-fill-function t)))
+			    (setq fill-column fc)
+			    (auto-fill-mode 1)))))
 		(setcar defaults keywords)
 		(develock-set-defaults defaults ad-do-it))
 	    ad-do-it))))))
@@ -1139,8 +1166,7 @@ Energize font-lock keywords for extraordinary level highlighting."
 		  (replace-match "^[\t ]*\\1" nil nil def)))))
 
 (defmacro develock-max-column ()
-  "Return nil or a number to limit that lines should not be longer than
-the value."
+  "Return the maximum column number from which lines won't be highlighted."
   '(let ((value (plist-get develock-max-column-plist major-mode)))
      (cond ((natnump value)
 	    value)
@@ -1174,10 +1200,11 @@ the value."
 		done)))))
 
 (defun develock-find-long-lines (limit)
-  "Function used to find a long line between the point and LIMIT.  If a
-long line is found, it returns non-nil and sets the `match-data' #0,
-#1 and #2 with the beginning and the end positions to be highlighted.
-This function is affected by the values of `develock-max-column-plist',
+  "Function used to find a long line between the point and LIMIT.
+If a long line is found, it returns non-nil and sets the `match-data'
+#0, #1 and #2 with the beginning and the end positions to be
+highlighted.  This function is affected by the values of
+`develock-max-column-plist',
 `develock-mode-ignore-long-message-header-list' and
 `develock-inhibit-highlighting-kinsoku-chars'."
   (let ((max-column (develock-max-column))
@@ -1261,9 +1288,10 @@ non-nil and sets beginning and ending points as the `match-data' #0,
 
 ;;;###autoload
 (defun develock-mode (arg)
-  "Toggle Develock mode.  With arg, turn Develock on if and only if arg
-is positive.  In Develock mode, highlighting leading and trailing
-whitespace, long lines and oddities."
+  "Toggle Develock mode.
+With ARG, turn Develock on if and only if ARG is positive.
+In Develock mode, highlighting leading and trailing whitespace,
+long lines and oddities."
   (interactive "P")
   (if (assq major-mode develock-keywords-alist)
       (cond ((and (not develock-mode)
@@ -1280,11 +1308,36 @@ whitespace, long lines and oddities."
 	     (if (interactive-p)
 		 (message "Develock is inhibited for this file")))
 	    (t
-	     (setq develock-mode (if (numberp arg)
-				     (> arg 0)
-				   (if font-lock-mode
-				       (not develock-mode)
-				     t)))
+	     (let ((oldmode develock-mode)
+		   fc)
+	       (setq develock-mode (if (numberp arg)
+				       (> arg 0)
+				     (if font-lock-mode
+					 (not develock-mode)
+				       t)))
+	       (if develock-mode
+		   (if (and (not oldmode)
+			    (setq fc (cdr (assq major-mode
+						develock-fill-column-alist))))
+		       (progn
+			 (set (make-local-variable
+			       'develock-original-fill-configuration)
+			      (cons (current-fill-column)
+				    (and auto-fill-function t)))
+			 (setq fill-column fc)
+			 (auto-fill-mode 1)))
+		 (if (and oldmode
+			  (local-variable-p
+			   'develock-original-fill-configuration
+			   (current-buffer))
+			  (car develock-original-fill-configuration))
+		     (progn
+		       (setq fill-column
+			     (car develock-original-fill-configuration))
+		       (auto-fill-mode
+			(if (cdr develock-original-fill-configuration)
+			    1
+			  0))))))
 	     (if font-lock-mode
 		 (progn
 		   (font-lock-mode 0)
@@ -1329,8 +1382,8 @@ This function is used for `lisp-interaction-mode-hook' exclusively."
 
 (defcustom develock-energize-functions-plist
   '(indent-region t lisp-indent-line t c-indent-line t newline-and-indent nil)
-  "Plist of functions and flags whether to be advised to remove useless
-whitespace."
+  "Plist of commands and flags; command is advised when flag is non-nil.
+When command is advised, it removes useless whitespace."
   :type
   '(list
     :convert-widget
@@ -1466,8 +1519,9 @@ IGNORE-POINT-POS are ignored."
 (eval-when-compile
   (defvar c-syntactic-indentation))
 
-(if (not (fboundp 'c-guess-basic-syntax))
-    (defalias 'c-guess-basic-syntax 'ignore))
+(eval-and-compile
+  (if (not (fboundp 'c-guess-basic-syntax))
+      (defalias 'c-guess-basic-syntax 'ignore)))
 
 (defun develock-c-indent-line (&optional syntax quiet ignore-point-pos)
   "Internal function used to advise some C indent functions."
